@@ -16,8 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: caps.c,v 1.24 2006/01/14 13:10:33 ovitters Exp $"
-#include "../config.h"
+#include <config.h>
 #include <stdlib.h>
 #include <glib.h>
 #include "caps.h"
@@ -42,7 +41,7 @@
 
 /* This list combined from the Linux termcap(5) man page, and
  * termcap_&_terminfo by Strang, Mui, and O'Reilly. */
-struct _vte_capability_quark _vte_terminal_capability_strings[] = {
+VTE_STATIC struct _vte_capability_quark _vte_terminal_capability_strings[] = {
 	{"!1", TRUE, 0},
 	{"!2", TRUE, 0},
 	{"!3", TRUE, 0},
@@ -357,12 +356,13 @@ struct _vte_capability_quark _vte_terminal_capability_strings[] = {
 	{"XF", FALSE, 0},
 	{"XN", FALSE, 0},
 
-	{NULL, FALSE, 0},
+
+	{"", FALSE, 0}
 };
 
 /* From some really old XTerm docs we had at the office, and an updated
  * version at Moy, Gildea, and Dickey. */
-struct _vte_capability_string _vte_xterm_capability_strings[] = {
+VTE_STATIC struct _vte_capability_string _vte_xterm_capability_strings[] = {
 	{ENQ, "return-terminal-status", 0},
 	{VT,  "vertical-tab", 0},
 	{FF,  "form-feed", 0},
@@ -431,6 +431,7 @@ struct _vte_capability_string _vte_xterm_capability_strings[] = {
 	{CSI "G", "cursor-character-absolute", 0},
 	{CSI "%dG", "cursor-character-absolute", 0},
 	{CSI ";H", "cursor-position", 0},
+	{CSI "%dH", "cursor-position", 0},
 	{CSI "%d;H", "cursor-position", 0},
 	{CSI ";%dH", "cursor-position", 0},
 	{CSI "%d;%dH", "cursor-position", 0},
@@ -571,17 +572,16 @@ void
 _vte_capability_init(void)
 {
 	unsigned int i;
-	GQuark quark;
-	for (i = 0; i < G_N_ELEMENTS(_vte_terminal_capability_strings); i++) {
+	for (i = 0; _vte_terminal_capability_strings[i].capability[0]; i++) {
 		const char *tmp;
+		GQuark quark;
 		tmp = _vte_terminal_capability_strings[i].capability;
-		if (tmp != NULL) {
-			quark = g_quark_from_static_string(tmp);
-			_vte_terminal_capability_strings[i].quark = quark;
-		}
+		quark = g_quark_from_static_string(tmp);
+		_vte_terminal_capability_strings[i].quark = quark;
 	}
 	for (i = 0; i < G_N_ELEMENTS(_vte_xterm_capability_strings); i++) {
 		const char *tmp;
+		GQuark quark;
 		tmp = _vte_xterm_capability_strings[i].value;
 		if (tmp != NULL) {
 			quark = g_quark_from_static_string(tmp);
