@@ -52,7 +52,6 @@
 
 G_BEGIN_DECLS
 
-#define VTE_CURSOR_OUTLINE		1
 #define VTE_PAD_WIDTH			1
 #define VTE_TAB_WIDTH			8
 #define VTE_LINE_WIDTH			1
@@ -62,15 +61,10 @@ G_BEGIN_DECLS
 #define VTE_COLOR_PLAIN_OFFSET		0
 #define VTE_COLOR_BRIGHT_OFFSET		8
 #define VTE_COLOR_DIM_OFFSET		16
-#define VTE_DEF_FG			256
-#define VTE_DEF_BG			257
-#define VTE_BOLD_FG			258
-#define VTE_DIM_FG			259
-#define VTE_DEF_HL                      260
-#define VTE_CUR_BG			261
+/* More color defines in ring.h */
 
-#define VTE_SATURATION_MAX		10000
 #define VTE_SCROLLBACK_INIT		100
+#define VTE_SATURATION_MAX		10000
 #define VTE_DEFAULT_CURSOR		GDK_XTERM
 #define VTE_MOUSING_CURSOR		GDK_LEFT_PTR
 #define VTE_TAB_MAX			999
@@ -91,41 +85,50 @@ G_BEGIN_DECLS
 #define VTE_MAX_PROCESS_TIME		100
 #define VTE_CELL_BBOX_SLACK		1
 
-/* The structure we use to hold characters we're supposed to display -- this
- * includes any supported visible attributes. */
-struct vte_charcell {
-	gunichar c;		/* The Unicode character. */
+#define VTE_UTF8_BPC                    (6) /* Maximum number of bytes used per UTF-8 character */
 
-	struct vte_charcell_attr {
-		guint32 columns: 2;	/* Number of visible columns
-					   (as determined
-					   by g_unicode_iswide(c)). */
-		guint32 fore: 9;	/* Index into color palette */
-		guint32 back: 9;	/* Index into color palette. */
+#define I_(string) (g_intern_static_string(string))
 
-		guint32 fragment: 1;	/* A continuation cell. */
-		guint32 standout: 1;	/* Single-bit attributes. */
-		guint32 underline: 1;
-		guint32 strikethrough: 1;
 
-		guint32 reverse: 1;
-		guint32 blink: 1;
-		guint32 half: 1;
-		guint32 bold: 1;
+typedef enum {
+        VTE_REGEX_GREGEX,
+        VTE_REGEX_VTE,
+        VTE_REGEX_UNDECIDED
+} VteRegexMode;
 
-		guint32 invisible: 1;
-		guint32 protect: 1;
-		guint32 alternate: 1;
+typedef enum {
+  VTE_REGEX_CURSOR_GDKCURSOR,
+  VTE_REGEX_CURSOR_GDKCURSORTYPE,
+  VTE_REGEX_CURSOR_NAME
+} VteRegexCursorMode;
 
-		/* 31 bits */
-	} attr;
-};
+/* The order is important */
+typedef enum {
+	MOUSE_TRACKING_NONE,
+	MOUSE_TRACKING_SEND_XY_ON_CLICK,
+	MOUSE_TRACKING_SEND_XY_ON_BUTTON,
+	MOUSE_TRACKING_HILITE_TRACKING,
+	MOUSE_TRACKING_CELL_MOTION_TRACKING,
+	MOUSE_TRACKING_ALL_MOTION_TRACKING
+} MouseTrackingMode;
 
 /* A match regex, with a tag. */
 struct vte_match_regex {
-	struct _vte_regex *reg;
 	gint tag;
-	GdkCursor *cursor;
+        VteRegexMode mode;
+        union { /* switched on |mode| */
+              struct {
+                    GRegex *regex;
+                    GRegexMatchFlags flags;
+              } gregex;
+              struct _vte_regex *reg;
+        } regex;
+        VteRegexCursorMode cursor_mode;
+        union {
+	       GdkCursor *cursor;
+               char *cursor_name;
+               GdkCursorType cursor_type;
+        } cursor;
 };
 
 /* The terminal's keypad/cursor state.  A terminal can either be using the
@@ -141,11 +144,6 @@ typedef struct _VteWordCharRange {
 	gunichar start, end;
 } VteWordCharRange;
 
-typedef struct _VteRowData {
-	GArray *cells;
-	guchar soft_wrapped: 1;
-} VteRowData;
-
 /* Terminal private data. */
 struct _VteTerminalPrivate {
 	/* Emulation setup data. */
@@ -174,22 +172,22 @@ struct _VteTerminalPrivate {
 	/* PTY handling data. */
 	const char *shell;		/* shell we started */
 	int pty_master;			/* pty master descriptor */
-	GIOChannel *pty_input;		/* master input watch */
+	GIOChannel *pty_channel;	/* master channel */
 	guint pty_input_source;
-	gboolean pty_input_active;
-	GIOChannel *pty_output;		/* master output watch */
 	guint pty_output_source;
+	gboolean pty_input_active;
 	GPid pty_pid;			/* pid of child using pty slave */
 	VteReaper *pty_reaper;
+        int child_exit_status;
 
 	/* Input data queues. */
 	const char *encoding;		/* the pty's encoding */
 	struct _vte_iso2022_state *iso2022;
 	struct _vte_incoming_chunk{
 		struct _vte_incoming_chunk *next;
-		gint len;
+		guint len;
 		guchar data[VTE_INPUT_CHUNK_SIZE
-			- sizeof(struct _vte_incoming_chunk *) - sizeof(gint)];
+			- 2 * sizeof(void *)];
 	} *incoming;			/* pending bytestream */
 	GArray *pending;		/* pending characters */
 	GSList *update_regions;
@@ -199,17 +197,16 @@ struct _VteTerminalPrivate {
 	glong max_input_bytes;
 
 	/* Output data queue. */
-	struct _vte_buffer *outgoing;	/* pending input characters */
+	VteBuffer *outgoing;	/* pending input characters */
 	VteConv outgoing_conv;
 
 	/* IConv buffer. */
-	struct _vte_buffer *conv_buffer;
+	VteBuffer *conv_buffer;
 
 	/* Screen data.  We support the normal screen, and an alternate
 	 * screen, which seems to be a DEC-specific feature. */
 	struct _VteScreen {
-		VteRing *row_data;	/* row data, arranged as a GArray of
-					   vte_charcell structures */
+		VteRing row_data[1];	/* buffer contents */
 		struct vte_cursor_position {
 			long row, col;
 		} cursor_current, cursor_saved;
@@ -228,22 +225,21 @@ struct _VteTerminalPrivate {
 		gboolean scrolling_restricted;
 		long scroll_delta;	/* scroll offset */
 		long insert_delta;	/* insertion offset */
-		struct vte_charcell defaults;	/* default characteristics
+		VteCell defaults;	/* default characteristics
 						   for insertion of any new
 						   characters */
-		struct vte_charcell color_defaults;	/* original defaults
+		VteCell color_defaults;	/* original defaults
 							   plus the current
 							   fore/back */
-		struct vte_charcell fill_defaults;	/* original defaults
+		VteCell fill_defaults;	/* original defaults
 							   plus the current
 							   fore/back with no
 							   character data */
-		struct vte_charcell basic_defaults;	/* original defaults */
+		gboolean alternate_charset;
 		gboolean status_line;
 		GString *status_line_contents;
 		gboolean status_line_changed;
 	} normal_screen, alternate_screen, *screen;
-	VteRowData *free_row;
 
 	/* Selection information. */
 	GArray *word_chars;
@@ -251,8 +247,7 @@ struct _VteTerminalPrivate {
 	gboolean selecting;
 	gboolean selecting_restart;
 	gboolean selecting_had_delta;
-	gboolean block_mode;
-	gboolean had_block_mode;
+	gboolean selection_block_mode;
 	char *selection;
 	enum vte_selection_type {
 		selection_type_char,
@@ -260,10 +255,10 @@ struct _VteTerminalPrivate {
 		selection_type_line
 	} selection_type;
 	struct selection_event_coords {
-		double x, y;
-	} selection_origin, selection_last, selection_restart_origin;
-	struct selection_cell_coords {
 		long x, y;
+	} selection_origin, selection_last;
+	struct selection_cell_coords {
+		long row, col;
 	} selection_start, selection_end;
 
 	/* Miscellaneous options. */
@@ -287,29 +282,33 @@ struct _VteTerminalPrivate {
 	gboolean scroll_on_keystroke;
 	long scrollback_lines;
 
+	/* Cursor shape */
+	VteTerminalCursorShape cursor_shape;
+
 	/* Cursor blinking. */
+        VteTerminalCursorBlinkMode cursor_blink_mode;
 	gboolean cursor_blink_state;
-	gboolean cursor_blinks;
-	guint cursor_blink_tag;
-	gint cursor_blink_timeout;
-	gint64 cursor_blink_time;
+	guint cursor_blink_tag;           /* cursor blinking timeout ID */
+        gint cursor_blink_cycle;          /* gtk-cursor-blink-time / 2 */
+	gint cursor_blink_timeout;        /* gtk-cursor-blink-timeout */
+        gboolean cursor_blinks;           /* whether the cursor is actually blinking */
+	gint64 cursor_blink_time;         /* how long the cursor has been blinking yet */
 	gboolean cursor_visible;
 
 	/* Input device options. */
 	time_t last_keypress_time;
-	gboolean mouse_send_xy_on_click;
-	gboolean mouse_send_xy_on_button;
-	gboolean mouse_hilite_tracking;
-	gboolean mouse_cell_motion_tracking;
-	gboolean mouse_all_motion_tracking;
+
+	int mouse_tracking_mode; /* this is of type MouseTrackingMode,
+				    but we need to guarantee its type. */
 	guint mouse_last_button;
-	gdouble mouse_last_x, mouse_last_y;
+	long mouse_last_x, mouse_last_y;
 	gboolean mouse_autohide;
 	guint mouse_autoscroll_tag;
 
 	/* State variables for handling match checks. */
 	char *match_contents;
 	GArray *match_attributes;
+        VteRegexMode match_regex_mode;
 	GArray *match_regexes;
 	char *match;
 	int match_tag;
@@ -323,7 +322,6 @@ struct _VteTerminalPrivate {
 	PangoFontDescription *fontdesc;
 	VteTerminalAntiAlias fontantialias;
 	gboolean fontdirty;
-	GtkSettings *connected_settings;
 
 	/* Data used when rendering the text which reflects server resources
 	 * and data, which should be dropped when unrealizing and (re)created
@@ -335,7 +333,7 @@ struct _VteTerminalPrivate {
 	gboolean cursor_color_set;
 	struct vte_palette_entry {
 		guint16 red, green, blue;
-	} palette[VTE_CUR_BG + 1];
+	} palette[VTE_PALETTE_SIZE];
 
 	/* Mouse cursors. */
 	gboolean mouse_cursor_visible;
@@ -379,37 +377,50 @@ struct _VteTerminalPrivate {
 	/* Obscured? state. */
 	GdkVisibilityState visibility_state;
 
+	/* Font stuff. */
 	gboolean has_fonts;
+	glong line_thickness;
+	glong underline_position;
+	glong strikethrough_position;
 };
 
 
 VteRowData *_vte_terminal_ensure_row(VteTerminal *terminal);
-void _vte_terminal_set_pointer_visible(VteTerminal *terminal, gboolean visible);
-void _vte_invalidate_all(VteTerminal *terminal);
-void _vte_invalidate_cells(VteTerminal *terminal,
-			   glong column_start, gint column_count,
-			   glong row_start, gint row_count);
-void _vte_invalidate_cell(VteTerminal *terminal, glong col, glong row);
-void _vte_invalidate_cursor_once(VteTerminal *terminal, gboolean periodic);
+VTE_STATIC  void _vte_terminal_set_pointer_visible(VteTerminal *terminal, gboolean visible);
+VTE_STATIC  void _vte_invalidate_all(VteTerminal *terminal);
+VTE_STATIC  void _vte_invalidate_cells(VteTerminal *terminal,
+				       glong column_start, gint column_count,
+				       glong row_start, gint row_count);
+VTE_STATIC  void _vte_invalidate_cell(VteTerminal *terminal, glong col, glong row);
+VTE_STATIC  void _vte_invalidate_cursor_once(VteTerminal *terminal, gboolean periodic);
 VteRowData * _vte_new_row_data(VteTerminal *terminal);
-VteRowData * _vte_new_row_data_sized(VteTerminal *terminal, gboolean fill);
-VteRowData * _vte_reset_row_data (VteTerminal *terminal, VteRowData *row, gboolean fill);
-void _vte_terminal_adjust_adjustments(VteTerminal *terminal);
-void _vte_terminal_queue_contents_changed(VteTerminal *terminal);
-void _vte_terminal_emit_text_deleted(VteTerminal *terminal);
-void _vte_terminal_emit_text_inserted(VteTerminal *terminal);
-gboolean _vte_terminal_insert_char(VteTerminal *terminal, gunichar c,
-			       gboolean force_insert_mode,
-			       gboolean invalidate_cells);
-void _vte_terminal_scroll_region(VteTerminal *terminal,
-				 long row, glong count, glong delta);
-void _vte_terminal_set_default_attributes(VteTerminal *terminal);
-void _vte_terminal_clear_tabstop(VteTerminal *terminal, int column);
-gboolean _vte_terminal_get_tabstop(VteTerminal *terminal, int column);
-void _vte_terminal_set_tabstop(VteTerminal *terminal, int column);
-void _vte_terminal_update_insert_delta(VteTerminal *terminal);
+VTE_STATIC  void _vte_terminal_adjust_adjustments(VteTerminal *terminal);
+VTE_STATIC  void _vte_terminal_queue_contents_changed(VteTerminal *terminal);
+VTE_STATIC  void _vte_terminal_emit_text_deleted(VteTerminal *terminal);
+VTE_STATIC  void _vte_terminal_emit_text_inserted(VteTerminal *terminal);
+VTE_STATIC  void _vte_terminal_cursor_down (VteTerminal *terminal);
+VTE_STATIC  gboolean _vte_terminal_insert_char(VteTerminal *terminal, gunichar c,
+					       gboolean force_insert_mode,
+					       gboolean invalidate_cells);
+VTE_STATIC  void _vte_terminal_scroll_region(VteTerminal *terminal,
+					     long row, glong count, glong delta);
+VTE_STATIC  void _vte_terminal_set_default_attributes(VteTerminal *terminal);
+VTE_STATIC  void _vte_terminal_clear_tabstop(VteTerminal *terminal, int column);
+VTE_STATIC  gboolean _vte_terminal_get_tabstop(VteTerminal *terminal, int column);
+VTE_STATIC  void _vte_terminal_set_tabstop(VteTerminal *terminal, int column);
+VTE_STATIC  void _vte_terminal_update_insert_delta(VteTerminal *terminal);
+VTE_STATIC  void _vte_terminal_cleanup_tab_fragments_at_cursor (VteTerminal *terminal);
+VTE_STATIC  void _vte_terminal_audible_beep(VteTerminal *terminal);
+VTE_STATIC  void _vte_terminal_visible_beep(VteTerminal *terminal);
+VTE_STATIC  void _vte_terminal_beep(VteTerminal *terminal);
 
-void _vte_terminal_inline_error_message(VteTerminal *terminal, const char *format, ...) G_GNUC_PRINTF(2,3);
+VTE_STATIC  void _vte_terminal_inline_error_message(VteTerminal *terminal, const char *format, ...) G_GNUC_PRINTF(2,3);
+
+/* vteseq.c: */
+VTE_STATIC  void _vte_terminal_handle_sequence(VteTerminal *terminal,
+					       const char *match_s,
+					       GQuark match,
+					       GValueArray *params);
 
 G_END_DECLS
 
