@@ -107,14 +107,6 @@ _vte_draw_init_default (struct _vte_draw *draw)
 }
 
 
-static void
-_vte_draw_update_requires_clear (struct _vte_draw *draw)
-{
-	draw->requires_clear = draw->impl->always_requires_clear
-			    || draw->bg_type != VTE_BG_SOURCE_NONE
-			    || draw->bg_opacity != 0xFFFF;
-}
-
 struct _vte_draw *
 _vte_draw_new (GtkWidget *widget)
 {
@@ -123,8 +115,6 @@ _vte_draw_new (GtkWidget *widget)
 	/* Create the structure. */
 	draw = g_slice_new0 (struct _vte_draw);
 	draw->widget = g_object_ref (widget);
-	draw->bg_type = VTE_BG_SOURCE_NONE;
-	draw->bg_opacity = 0xffff; 
 
 	/* Allow the user to specify her preferred backends */
 	if (!_vte_draw_init_user (draw) &&
@@ -135,7 +125,7 @@ _vte_draw_new (GtkWidget *widget)
 		draw->impl = &_vte_draw_skel;
 	}
 
-	_vte_draw_update_requires_clear (draw);
+	draw->requires_clear = draw->impl->always_requires_clear;
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
 			"draw_new (%s)\n", draw->impl->name);
@@ -225,18 +215,14 @@ _vte_draw_end (struct _vte_draw *draw)
 }
 
 void
-_vte_draw_set_background_opacity (struct _vte_draw *draw,
-				  guint16 opacity)
+_vte_draw_set_background_solid(struct _vte_draw *draw,
+			       GdkColor *color,
+			       guint16 opacity)
 {
-	draw->bg_opacity = opacity;
-	_vte_draw_update_requires_clear (draw);
-}
+	draw->requires_clear = draw->impl->always_requires_clear || opacity != 0xFFFF;
 
-void
-_vte_draw_set_background_color (struct _vte_draw *draw,
-			        GdkColor *color)
-{
-	draw->bg_color = *color;
+	if (draw->impl->set_background_solid)
+		draw->impl->set_background_solid (draw, color, opacity);
 }
 
 void
@@ -247,14 +233,26 @@ _vte_draw_set_background_image (struct _vte_draw *draw,
 			        const GdkColor *color,
 			        double saturation)
 {
-	draw->bg_type = type;
-	_vte_draw_update_requires_clear (draw);
+	if (type != VTE_BG_SOURCE_NONE)
+		draw->requires_clear = TRUE;
 
 	if (draw->impl->set_background_image)
 		draw->impl->set_background_image (draw, type, pixbuf, filename,
 						  color, saturation);
 }
 
+void
+_vte_draw_set_background_scroll (struct _vte_draw *draw,
+				 gint x, gint y)
+{
+	_vte_debug_print (VTE_DEBUG_DRAW,
+			"draw_set_scroll (%d, %d)\n",
+			x, y);
+
+	if (draw->impl->set_background_scroll)
+		draw->impl->set_background_scroll (draw, x, y);
+}
+
 gboolean
 _vte_draw_requires_clear (struct _vte_draw *draw)
 {
@@ -283,7 +281,7 @@ _vte_draw_clear (struct _vte_draw *draw, gint x, gint y, gint width, gint height
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_clear (%d, %d, %d, %d)\n",
 			  x,y,width, height);
 
-	draw->impl->clear (draw, x, y, width, height);
+	(draw->impl->clear) (draw, x, y, width, height);
 }
 
 void
@@ -314,12 +312,13 @@ _vte_draw_get_text_metrics(struct _vte_draw *draw,
 }
 
 int
-_vte_draw_get_char_width (struct _vte_draw *draw, gunichar c, int columns)
+_vte_draw_get_char_width (struct _vte_draw *draw, vteunistr c, int columns,
+			  gboolean bold)
 {
 	int width = 0;
 
 	if (draw->impl->get_char_width)
-		width = draw->impl->get_char_width (draw, c, columns);
+		width = draw->impl->get_char_width (draw, c, columns, bold);
 
 	if (width == 0)
 		_vte_draw_get_text_metrics (draw, &width, NULL, NULL);
@@ -330,7 +329,7 @@ _vte_draw_get_char_width (struct _vte_draw *draw, gunichar c, int columns)
 void
 _vte_draw_text (struct _vte_draw *draw,
 	       struct _vte_draw_text_request *requests, gsize n_requests,
-	       GdkColor *color, guchar alpha)
+	       GdkColor *color, guchar alpha, gboolean bold)
 {
 	g_return_if_fail (draw->started == TRUE);
 	g_return_if_fail (draw->impl->draw_text != NULL);
@@ -343,43 +342,59 @@ _vte_draw_text (struct _vte_draw *draw,
 			g_string_append_unichar (string, requests[n].c);
 		}
 		str = g_string_free (string, FALSE);
-		g_printerr ("draw_text (\"%s\", len=%"G_GSIZE_FORMAT", color=(%d,%d,%d,%d))\n",
+		g_printerr ("draw_text (\"%s\", len=%"G_GSIZE_FORMAT", color=(%d,%d,%d,%d), %s)\n",
 				str, n_requests, color->red, color->green, color->blue,
-				alpha);
+				alpha, bold ? "bold" : "normal");
 		g_free (str);
 	}
 
-	draw->impl->draw_text (draw, requests, n_requests, color, alpha);
+	draw->impl->draw_text (draw, requests, n_requests, color, alpha, bold);
+
+	/* handle fonts that lack a bold face by double-striking */
+	if (bold && !(draw->impl->has_bold && draw->impl->has_bold (draw))) {
+		int i;
+
+		/* Take a step to the right. */
+		for (i = 0; i < n_requests; i++) {
+			requests[i].x++;
+		}
+		draw->impl->draw_text (draw, requests, n_requests, color, alpha, FALSE);
+		/* Now take a step back. */
+		for (i = 0; i < n_requests; i++) {
+			requests[i].x--;
+		}
+	}
 }
 
 gboolean
 _vte_draw_char (struct _vte_draw *draw,
 	       struct _vte_draw_text_request *request,
-	       GdkColor *color, guchar alpha)
+	       GdkColor *color, guchar alpha, gboolean bold)
 {
 	gboolean has_char;
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
-			"draw_char ('%c', color=(%d,%d,%d,%d))\n",
+			"draw_char ('%c', color=(%d,%d,%d,%d), %s)\n",
 			request->c,
 			color->red, color->green, color->blue,
-			alpha);
+			alpha, bold ? "bold" : "normal");
 
-	has_char =_vte_draw_has_char (draw, request->c);
+	has_char =_vte_draw_has_char (draw, request->c, bold);
 	if (has_char)
-		_vte_draw_text (draw, request, 1, color, alpha);
+		_vte_draw_text (draw, request, 1, color, alpha, bold);
 
 	return has_char;
 }
 gboolean
-_vte_draw_has_char (struct _vte_draw *draw, gunichar c)
+_vte_draw_has_char (struct _vte_draw *draw, vteunistr c, gboolean bold)
 {
 	gboolean has_char = TRUE;
 
-	_vte_debug_print (VTE_DEBUG_DRAW, "draw_has_char ('%c')\n", c);
+	_vte_debug_print (VTE_DEBUG_DRAW, "draw_has_char ('0x%04X', %s)\n", c,
+			  bold ? "bold" : "normal");
 
 	if (draw->impl->has_char)
-		has_char = draw->impl->has_char (draw, c);
+		has_char = draw->impl->has_char (draw, c, bold);
 
 	return has_char;
 }
@@ -427,14 +442,3 @@ _vte_draw_draw_rectangle (struct _vte_draw *draw,
 		}
 	}
 }
-
-void
-_vte_draw_set_scroll (struct _vte_draw *draw, gint x, gint y)
-{
-	_vte_debug_print (VTE_DEBUG_DRAW,
-			"draw_set_scroll (%d, %d)\n",
-			x, y);
-
-	draw->scrollx = x;
-	draw->scrolly = y;
-}
